/**
 * IMPOTANT
 * The config file "advancedtagmanager_simpletags.cfg" should be ranked by the flags required.
 * It provides a fast and easy configuration by just specifying the flags, but if two flags
 * of the tags are the same or anything else that would make the user get the incorrect flags
 * you must use the "advancedtagmanager_tags.cfg" where you specify the STEAMID2 of the user.
 * 
 * "advancedtagmanager_tags.cfg" will always take priority over "advancedtagmanager_simpletags.cfg"
 * because it is a more targeted way of managing the tag.
 */

/**
 * If you want to use LevelsRanks then uncomment the next line */
//#define LEVELSRANK_ENABLED
/** 
 * If you have modified the LevelsRanks databases's names you must modify them accordingly down here */
#define LR_BASE                     "lr_base"
#define LVL_BASE                    "lvl_base"
/**
 * If you have modified the LevelsRanks translation file name you must replace accordingly down here. */
#define LR_CORE_RANKS_TRANSLATION   "lr_core_ranks.phrases"

// Plugin originally was: https://forums.alliedmods.net/showthread.php?t=299351
#pragma semicolon 1 

/* Sourcemod for most natives and functions */
#include <sourcemod> 
/* Client prefs for storing cookies */
#include <clientprefs>
/* Simple-ChatProcessor for modifying the chat behaviour */
#include <scp>
/* Database include for connecting to the levels_rank database */
#include <dbi>
public Plugin myinfo = 
{
	name = "[(?) CSGO / CSS] Advanced chat tag manager", 
	author = "bl4ze", 
	description = "An advanced chat tag manager", 
	version = "1.5_10.02.2024", 
	url = "https://steamcommunity.com/id/probabilcodat/"
};
/* Multicolors for well, colors :O */
#include <multicolors>
/* LevelsRanks for handling ranks in chat */
#if defined LEVELSRANK_ENABLED
#include <lvl_ranks>
#endif 

/* Memory defines (the ones starting with LENGTH_OF do not include the null terminator)*/
#define MAX_CONFIG_ENTRIES          100
#define GET_RANK                    0
#define GET_TAG                     1
#define LENGTH_OF_TAG               24
#define LENGTH_OF_RANK_PHRASE       8
#define LENGTH_OF_FLAG              16
#define LENGTH_OF_COLOR             32
#define LENGTH_OF_ENGINE_STEAMID    32
#define LENGTH_OF_STEAM2_STEAMID    32
#define COOKIE_MAX_LENGTH           (LENGTH_OF_TAG + LENGTH_OF_RANK_PHRASE + 2)
#define PRIORITY_TAG_MAX_IDS        16

// Handles
Cookie      g_ckTagCookie; 
ConVar      g_cvPluginState;
#if defined LEVELSRANK_ENABLED
ConVar      g_cvTrimName;
Database    g_dbLevelRank;
// Database additional data
bool        g_bConnected;
#endif

// Tag related
char        g_szTags[MAX_CONFIG_ENTRIES][LENGTH_OF_TAG + 1],
            g_szFlags[MAX_CONFIG_ENTRIES][LENGTH_OF_FLAG + 1],
            g_szTagColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
            g_szNameColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
            g_szTextColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
#if defined LEVELSRANK_ENABLED
            g_szRanks[MAX_CONFIG_ENTRIES][LENGTH_OF_RANK_PHRASE + 1],
            g_szRankColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
            g_szRankNameColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
            g_szRankTextColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
#endif
            g_szPriorityTags[MAX_CONFIG_ENTRIES][LENGTH_OF_TAG + 1],
            g_szPriorityTagColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
            g_szPriorityNameColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
            g_szPriorityTextColors[MAX_CONFIG_ENTRIES][LENGTH_OF_COLOR + 1],
            g_szSteam2SteamIDs[MAX_CONFIG_ENTRIES][PRIORITY_TAG_MAX_IDS][LENGTH_OF_ENGINE_STEAMID + 1];
int         g_iTagsLoaded                   = 0,
#if defined LEVELSRANK_ENABLED
            g_iRanksLoaded                  = 0,
#endif
            g_iPriorityTagsLoaded           = 0;
bool        g_bRanksDisabled                = false;

/**
 * SOURCEMOD NATIVES
 */
public void OnPluginStart()
{
    // Register the cookie in which we store every player's tag
    g_ckTagCookie = RegClientCookie("advtm_chattag", "A cookie for saving clients's tags", CookieAccess_Private);

    // Create and register admin commands and cvars
    g_cvPluginState = CreateConVar("sm_advtm_enable",   "1", "Enable or disable the plugin",   _, true, 0.0, true, 1.0);
#if defined LEVELSRANK_ENABLED
    g_cvTrimName    = CreateConVar("sm_advtm_trim",     "1", "Try to fit rank, tag and name before the ':' (only affects players with both)",   _, true, 0.0, true, 1.0);
#endif
    RegAdminCmd("sm_advtm_reload",          Command_Reload,     ADMFLAG_RCON, "Reloads all of the configs and reassigns tags");
    RegAdminCmd("sm_advtm_getcookie",       Command_GetCookie,  ADMFLAG_RCON, "Get the advtm_chattag cookie of a client");
    RegAdminCmd("sm_advtm_setcookie",       Command_SetCookie,  ADMFLAG_RCON, "Set the advtm_chattag cookie of a client (dont use unless necessary)");

    // Load LevelsRanks translations (https://forums.alliedmods.net/showthread.php?t=318782) and the tag data from the config file
#if defined LEVELSRANK_ENABLED
    LoadTranslations(LR_CORE_RANKS_TRANSLATION);
#endif 

    LoadTranslations("common.phrases");
    AutoExecConfig(true, "plugin.advanced_tagmanager");
    LoadConfiguration();
}
public void OnPluginEnd() 
{
#if defined LEVELSRANK_ENABLED
    LR_Unhook(LR_OnLevelChangedPost, OnClientLevelChange);
#endif 
}
// We do the same thing twice here and in OnClientCookiesCached because one can be called before the other fully loads
public void OnClientPostAdminCheck(int client)  
{
    AssignCorrespondingTag(client, false);
}
public void OnClientCookiesCached(int client) 
{
    AssignCorrespondingTag(client, _, false);

#if defined LEVELSRANK_ENABLED
    if (!g_bRanksDisabled && g_bConnected) // Get his rank
    {
        char    szCookieValue[COOKIE_MAX_LENGTH],
                szCookie2D[2][LENGTH_OF_TAG + 1];
        g_ckTagCookie.Get(client, szCookieValue, sizeof(szCookieValue));
        ExplodeString(szCookieValue, "!", szCookie2D, 2, LENGTH_OF_TAG + 1);
        // Get the engine steam id (used by cookies and pack it)
        char szEngineSteamID[LENGTH_OF_ENGINE_STEAMID + 1], szSteam2SteamID[LENGTH_OF_STEAM2_STEAMID + 1];
        GetClientAuthId(client, AuthId_Engine, szEngineSteamID, sizeof(szEngineSteamID));
        DataPack dpSteamID = new DataPack();
        dpSteamID.WriteCell(GetClientSerial(client));
        dpSteamID.WriteString(szEngineSteamID); 

        // Get the steam2 steam id (used by LevelsRank)
        GetClientAuthId(client, AuthId_Steam2, szSteam2SteamID, sizeof(szSteam2SteamID));   

        // Get the rank of the player
        char szQuery[256];						
        FormatEx(szQuery, sizeof(szQuery), "SELECT rank FROM '%s' WHERE steam = \"%s\"", LVL_BASE, szSteam2SteamID);
        g_dbLevelRank.Query(SQL_GetRank, szQuery, dpSteamID);
    } 
#endif
}

/**
 * ADMIN COMMANDS
 */
public Action Command_Reload(int client, int args)
{
    LoadConfiguration();
    CReplyToCommand(client, "%s", "\x04[Advanced-TagManager]\x01 Reloaded.");   

    return Plugin_Handled;
}

public Action Command_GetCookie(int client, int args) 
{
    CookieActionHandler(client, args, false);
    return Plugin_Handled;
}
public Action Command_SetCookie(int client, int args) 
{
    CookieActionHandler(client, args, true);
    return Plugin_Handled;
}
public Action OnChatMessage(int &client, Handle hRecipients, char[] szName, char[] szMessage)
{
    if(g_cvPluginState.BoolValue && (MaxClients >= client > 0))
    {
        if(szMessage[0] == '/' || szMessage[0] == '@')
        	return Plugin_Continue;
		
        char    szClientName[MAXLENGTH_NAME],
                szClientMessage[MAXLENGTH_MESSAGE];
        strcopy(szClientName,       sizeof(szClientName),       szName);
        strcopy(szClientMessage,    sizeof(szClientMessage),    szMessage); 

        // Get the tag and rank
        char    szCookieValue[COOKIE_MAX_LENGTH],
                szCookie2D[2][LENGTH_OF_TAG + 1];
        g_ckTagCookie.Get(client, szCookieValue, sizeof(szCookieValue));
        ExplodeString(szCookieValue, "!", szCookie2D, 2, LENGTH_OF_TAG + 1);

        // Color related 
        char    szTagColor[LENGTH_OF_COLOR + 1], 
                szNameColor[LENGTH_OF_COLOR + 1], 
                szTextColor[LENGTH_OF_COLOR + 1];       
#if defined LEVELSRANK_ENABLED
        char    szRankColor[LENGTH_OF_COLOR + 1]; 
        FindRankColor(szCookie2D[GET_RANK], szRankColor,   szNameColor, szTextColor);
#endif 
        FindTagColor(szCookie2D[GET_TAG],   szTagColor,    szNameColor, szTextColor);

        // Format the message acordingly
        if (g_bRanksDisabled) 
        {
            if (szCookie2D[GET_TAG][0] == '\0') // Also does not have a tag wtf bro 
            {
                // Format:  szName      = name
                //          szMessage   = message
                FormatEx(szName,    MAXLENGTH_NAME,     "%s%s", szNameColor, szClientName);
                FormatEx(szMessage, MAXLENGTH_MESSAGE,  "%s%s", szTextColor, szClientMessage);
            }
            else 
            {
                // Format:  szName      = tag + name
                //          szMessage   = message
                FormatEx(szName,    MAXLENGTH_NAME,     "%s%s %s%s",    szTagColor,     szCookie2D[GET_TAG], szNameColor, szClientName);
                FormatEx(szMessage, MAXLENGTH_MESSAGE,  "%s%s",         szTextColor,    szClientMessage);
            }
        }
#if defined LEVELSRANK_ENABLED
        else 
        {
            if (szCookie2D[GET_RANK][0] == '\0' && szCookie2D[GET_TAG][0] == '\0') // Does not have a rank somehow neither a tag
            {
                // Format:  szName      = name
                //          szMessage   = message
                char szSteam2SteamID[LENGTH_OF_STEAM2_STEAMID + 1];
                GetClientAuthId(client, AuthId_Steam2, szSteam2SteamID, sizeof(szSteam2SteamID));
                LogError("Player %s has with SteamID2 %s has neither a tag, neither a rank.", szName, szSteam2SteamID);

                FormatEx(szName,    MAXLENGTH_NAME,     "%s%s", szNameColor, szClientName);
                FormatEx(szMessage, MAXLENGTH_MESSAGE,  "%s%s", szTextColor, szClientMessage);
            }
            else if (szCookie2D[GET_RANK][0] == '\0') // Does not have a rank
            {
                // Format:  szName      = tag + name
                //          szMessage   = message
                FormatEx(szName,    MAXLENGTH_NAME,     "%s%s %s%s",    szTagColor,     szCookie2D[GET_TAG], szNameColor, szClientName);
                FormatEx(szMessage, MAXLENGTH_MESSAGE,  "%s%s",         szTextColor,    szClientMessage);
            } 
            else if (szCookie2D[GET_TAG][0] == '\0') // Does not have a tag
            {
                // Format:  szName      = rank + name
                //          szMessage   = message
                FormatEx(szName,    MAXLENGTH_NAME,     "%s[%T] %s%s",  szRankColor, szCookie2D[GET_RANK], LANG_SERVER, szNameColor, szClientName);
                FormatEx(szMessage, MAXLENGTH_MESSAGE,  "%s%s",         szTextColor, szClientMessage);
            }
            else // Has both of them 
            {
                if (g_cvTrimName.BoolValue) 
                {
                    // Format:  szName      = rank + tag + name
                    //          szMessage   = message
                    FormatEx(szName,    MAXLENGTH_NAME,     "%s[%T] %s%s %s%s", szRankColor, szCookie2D[GET_RANK], LANG_SERVER, szTagColor, szCookie2D[GET_TAG], szNameColor, szClientName);
                    FormatEx(szMessage, MAXLENGTH_MESSAGE,  "%s%s",           szTextColor, szClientMessage);
                }
                else 
                {
                    // Format:  szName      = rank + tag
                    //          szMessage   = name + message
                    FormatEx(szName,                MAXLENGTH_NAME,                     "%s[%T] %s%s",      szRankColor, szCookie2D[GET_RANK], LANG_SERVER, szTagColor, szCookie2D[GET_TAG]);
                    FormatEx(szMessage,             MAXLENGTH_MESSAGE,                  "%s%s \x01> %s%s",  szNameColor, szClientName,  szTextColor, szClientMessage);
                }
            }
        }
#endif 

        CFormatColor(szMessage, MAXLENGTH_MESSAGE);
        CFormatColor(szName,    MAXLENGTH_NAME);

        return Plugin_Changed;
    }

    return Plugin_Continue;
} 

/**
 * LOADING CONFIGURATION 
 */
public void LoadConfiguration()
{
    char szPath[PLATFORM_MAX_PATH];
    BuildPath(Path_SM, szPath, sizeof(szPath), "configs/advancedtagmanager_simpletags.cfg");
    // Load tags
    KeyValues kvTags = CreateKeyValues("AdvancedTagManager");
    if (kvTags.ImportFromFile(szPath) && kvTags.GotoFirstSubKey())
    {
        g_iTagsLoaded    = 0;
        do
        {
            kvTags.GetString("tag",         g_szTags[g_iTagsLoaded],                sizeof(g_szTags));
            kvTags.GetString("flag",        g_szFlags[g_iTagsLoaded],               sizeof(g_szFlags));
            kvTags.GetString("tag_color",   g_szTagColors[g_iTagsLoaded],           sizeof(g_szTagColors),      "\x01");
            kvTags.GetString("name_color",  g_szNameColors[g_iTagsLoaded],          sizeof(g_szNameColors),     "\x03");
            kvTags.GetString("text_color",  g_szTextColors[g_iTagsLoaded],          sizeof(g_szTextColors),     "\x01");
			
            ++g_iTagsLoaded;
		} while (kvTags.GotoNextKey());
    }
    else
        SetFailState("[Advanced-TagManager] Error in parsing file advancedtagmanager_simpletags.cfg.");
        
    Safe_CloseHandle(kvTags);

    BuildPath(Path_SM, szPath, sizeof(szPath), "configs/advancedtagmanager_tags.cfg");
    KeyValues kvPriorityTags = CreateKeyValues("AdvancedTagManager");
    if (kvPriorityTags.ImportFromFile(szPath) && kvPriorityTags.GotoFirstSubKey()) 
    {
        g_iPriorityTagsLoaded = 0;
        char szSteamIdx[3]; // 16 has two chars + null terminator
        int iSteamIdx;
        do 
        {
            // Has STEAMID, take in account this 
            if (kvPriorityTags.GotoFirstSubKey()) 
            {
                for (iSteamIdx = 0; iSteamIdx < PRIORITY_TAG_MAX_IDS; ++iSteamIdx) 
                {
                    IntToString(iSteamIdx, szSteamIdx, sizeof(szSteamIdx));
                    kvPriorityTags.GetString(szSteamIdx, g_szSteam2SteamIDs[g_iPriorityTagsLoaded][iSteamIdx], LENGTH_OF_STEAM2_STEAMID + 1);

                    if (g_szSteam2SteamIDs[g_iPriorityTagsLoaded][iSteamIdx][0] == '\0') 
                        break;
                }

                kvPriorityTags.GoBack();

                kvPriorityTags.GetString("tag",         g_szPriorityTags[g_iPriorityTagsLoaded],                 sizeof(g_szPriorityTags));
                kvPriorityTags.GetString("tag_color",   g_szPriorityTagColors[g_iPriorityTagsLoaded],            sizeof(g_szPriorityTagColors),         "\x01");
                kvPriorityTags.GetString("name_color",  g_szPriorityNameColors[g_iPriorityTagsLoaded],           sizeof(g_szPriorityNameColors),        "\x03");
                kvPriorityTags.GetString("text_color",  g_szPriorityTextColors[g_iPriorityTagsLoaded],           sizeof(g_szPriorityTextColors),        "\x01");
                
                ++g_iPriorityTagsLoaded;
            } 
        } while (kvPriorityTags.GotoNextKey());
    }
    else 
        LogMessage("[Advanced-TagManager] Could not find advancedtagmanager_tags.cfg.");
    
    Safe_CloseHandle(kvPriorityTags);

#if defined LEVELSRANK_ENABLED
    BuildPath(Path_SM, szPath, sizeof(szPath), "configs/advancedtagmanager_ranks.cfg");
    // Load rank colors
    KeyValues kvRanks = CreateKeyValues("AdvancedTagManager");
    if (kvRanks.ImportFromFile(szPath) && kvRanks.GotoFirstSubKey()) 
    {
        g_bRanksDisabled    = false;
        g_iRanksLoaded      = 0;
        do 
        {
            kvRanks.GetString("rank_translation_phrase",    g_szRanks[g_iRanksLoaded],          sizeof(g_szRanks));
            kvRanks.GetString("rank_color",                 g_szRankColors[g_iRanksLoaded],     sizeof(g_szRankColors),     "{lightgreen}");
            kvRanks.GetString("name_color",                 g_szRankNameColors[g_iRanksLoaded], sizeof(g_szRankNameColors), "\x03");
            kvRanks.GetString("text_color",                 g_szRankTextColors[g_iRanksLoaded], sizeof(g_szRankTextColors), "\x01");

            ++g_iRanksLoaded;
        } while (kvRanks.GotoNextKey());
    }
    else 
    {
        g_bRanksDisabled    = true;
        LogError("[Advanced-TagManager] Could not find advancedtagmanager_ranks.cfg, ranks disabled");
    }
    Safe_CloseHandle(kvRanks);
#endif 
#if !defined LEVELSRANK_ENABLED  
    g_bRanksDisabled        = true;
#endif 

    // Remove tag if he does not have access to any or give him new tag that he has access to 
    char    szCookieValue[COOKIE_MAX_LENGTH],
            szCookie2D[2][LENGTH_OF_TAG + 1],
            szEngineSteamID[LENGTH_OF_ENGINE_STEAMID + 1]; 
    bool bValidTag; 
    for (int client = 1; client <= MaxClients; ++client) 
    {
        bValidTag = AssignCorrespondingTag(client);
        if (!bValidTag && AreClientCookiesCached(client) && IsClientAuthorized(client)) 
        {
            // Remove only the tag and update the cookie
            GetClientAuthId(client, AuthId_Engine, szEngineSteamID, sizeof(szEngineSteamID)); 
            FormatEx(szCookieValue, sizeof(szCookieValue), "%s!", szCookie2D[GET_RANK]);
            g_ckTagCookie.SetByAuthId(szEngineSteamID, szCookieValue);
        }
    }

#if defined LEVELSRANK_ENABLED
    if(!g_bRanksDisabled && !g_bConnected)  // If not disabled and we do not already have a connection open one :O
    {
        // Try and connect to the LevelsRank Database
        KeyValues kvDBInfo  = new KeyValues(NULL_STRING, "driver", "sqlite");
        kvDBInfo.SetString("database", LR_BASE);
        char szConnectionError[64];
        g_dbLevelRank = SQL_ConnectCustom(kvDBInfo, szConnectionError, sizeof(szConnectionError), false);
        if(g_dbLevelRank == INVALID_HANDLE)
            LogError("Could not connect to database: %s", szConnectionError);
        
        g_bConnected        = g_dbLevelRank != INVALID_HANDLE; 
        kvDBInfo.Close();

        if (g_bConnected) 
            LR_Hook(LR_OnLevelChangedPost, OnClientLevelChange);
    } 
    else if (g_bRanksDisabled && g_bConnected) // We have connection but they are disabled, Unhook the experience change
        LR_Unhook(LR_OnLevelChangedPost, OnClientLevelChange);
    else if (!g_bRanksDisabled && g_bConnected) // Update the rank
    {
        LR_RefreshConfigs();
        for (int i = 1; i <= MaxClients; ++i) 
            if (IsClientConnected(i) && IsClientInGame(i) && !IsFakeClient(i))
                FakeClientCommandEx(i, "sm_lvl"); // Opening the menu is calling "CheckRank" in LevelsRanks internally
    }
#endif 
} 

/**
 * LEVELSRANK SQL 
 */
#if defined LEVELSRANK_ENABLED
// Post hook
public void OnClientLevelChange(int iClient, int iNewLevel, int iOldLevel) 
{
    // Get the cookie values and separate the rank from the tag
    char    szCookieValue[COOKIE_MAX_LENGTH],
            szCookie2D[2][LENGTH_OF_TAG + 1];
    g_ckTagCookie.Get(iClient, szCookieValue, sizeof(szCookieValue));
    ExplodeString(szCookieValue, "!", szCookie2D, 2, LENGTH_OF_TAG + 1);    

    // Update the rank translation phrase
    FormatEx(szCookieValue, sizeof(szCookieValue), "rank_%d!%s", iNewLevel, szCookie2D[GET_TAG]);
    char szEngineSteamID[LENGTH_OF_ENGINE_STEAMID + 1];
    GetClientAuthId(iClient, AuthId_Engine, szEngineSteamID, sizeof(szEngineSteamID));
    g_ckTagCookie.SetByAuthId(szEngineSteamID, szCookieValue);  
}
public void SQL_Error(Database dbDatabase, DBResultSet dbrResults, const char[] szError, int iData)
{
    if (dbrResults == null) 
    	LogError("SQL_Error: %s", szError);
}
public void SQL_GetRank(Database dbDatabase, DBResultSet dbrResults, const char[] szError, DataPack dpSteamID)
{
    if (dbrResults == null)
	{
		LogError("Something went wrong: %s", szError);
		return;
	}
    int client;

    dpSteamID.Reset();
    client = GetClientFromSerial(dpSteamID.ReadCell());
    if (client == 0) {
        // Disconnected, cannot set new cookie
        Safe_CloseHandle(dpSteamID);
        return;
    }

    char    szCookieValue[COOKIE_MAX_LENGTH],
            szRank[LENGTH_OF_RANK_PHRASE + 1],
            szEngineSteamID[LENGTH_OF_ENGINE_STEAMID + 1],
            szCookie2D[2][LENGTH_OF_TAG + 1];
    dpSteamID.ReadString(szEngineSteamID, sizeof(szEngineSteamID));

    if (dbrResults.FetchRow())
    {
        g_ckTagCookie.Get(client, szCookieValue, sizeof(szCookieValue));
        ExplodeString(szCookieValue, "!", szCookie2D, 2, LENGTH_OF_TAG + 1);

        FormatEx(szRank, sizeof(szRank), "rank_%d", dbrResults.FetchInt(0));
        FormatEx(szCookieValue, sizeof(szCookieValue), "%s!%s", szRank, szCookie2D[GET_TAG]);

        g_ckTagCookie.SetByAuthId(szEngineSteamID, szCookieValue);
    }

    Safe_CloseHandle(dpSteamID);
} 
#endif 


/**
 * Helpful functions
 */
void FindTagColor(char[] szTag, char[] szTagColor, char[] szNameColor, char[] szTextColor)
{
    int i = 0;
    for (; i < g_iPriorityTagsLoaded; ++i) 
    {
        if (StrEqual(g_szPriorityTags[i], szTag)) 
        {
            strcopy(szTagColor,     sizeof(g_szPriorityTagColors),   g_szPriorityTagColors[i]);
            strcopy(szNameColor,    sizeof(g_szPriorityNameColors),  g_szPriorityNameColors[i]);
            strcopy(szTextColor,    sizeof(g_szTextColors),         g_szTextColors[i]);
            return;
        }
    }
    for (i = 0; i < g_iTagsLoaded; ++i)
	{
		if (StrEqual(g_szTags[i], szTag))
		{
			strcopy(szTagColor,     sizeof(g_szTagColors),  g_szTagColors[i]);
			strcopy(szNameColor,    sizeof(g_szNameColors), g_szNameColors[i]);
			strcopy(szTextColor,    sizeof(g_szTextColors), g_szTextColors[i]);
			return;
		}
	}
}
#if defined LEVELSRANK_ENABLED
void FindRankColor(char[] szRank, char[] szRankColor, char[] szNameColor, char[] szTextColor) 
{
    for (int i = 0; i < g_iRanksLoaded; ++i) 
    {
        if (StrEqual(g_szRanks[i], szRank)) 
        {
            strcopy(szRankColor,    sizeof(g_szRankColors),     g_szRankColors[i]);
            strcopy(szNameColor,    sizeof(g_szRankNameColors), g_szRankNameColors[i]);
            strcopy(szTextColor,    sizeof(g_szRankTextColors), g_szRankTextColors[i]);
            return;
        }
    }
} 
#endif
void Safe_CloseHandle(Handle hndl) 
{
    if (hndl != null && hndl != INVALID_HANDLE) 
        CloseHandle(hndl);

    hndl = null;
}

/**
 * AssignCorrespondingTag
 * 
 * Assigns the first tag that the client has access to and returns true on tag found
 * 
 * @param client            Client index
 * @param bAuthCheck        Should check if client is auth
 * @param bCookieCheck      Should check if client's cookies are cached
 * 
 * @return                  True on tag assigned, false on error/tag not found
 */
bool AssignCorrespondingTag(int client, bool bAuthCheck = true, bool bCookieCheck = true) 
{
// Get the cookies values ande separate the rank from the tag
    char    szCookieValue[COOKIE_MAX_LENGTH],
            szCookie2D[2][LENGTH_OF_TAG + 1];
    g_ckTagCookie.Get(client, szCookieValue, sizeof(szCookieValue));
    ExplodeString(szCookieValue, "!", szCookie2D, 2, LENGTH_OF_TAG + 1);
    bool bValidTag = false;
    
    if (IsClientConnected(client) == false || IsFakeClient(client))             return false;
    if (bCookieCheck            && AreClientCookiesCached(client) == false)     return false;
    if (bAuthCheck              && IsClientAuthorized(client) == false)         return false;

    if (szCookie2D[GET_TAG][0] == '\0') 
    {
        // Check for priority tags first 
        bool bPriorityFound = false;
        char szSteam2SteamID[LENGTH_OF_STEAM2_STEAMID + 1];
        if (g_iPriorityTagsLoaded)
            GetClientAuthId(client, AuthId_Steam2, szSteam2SteamID, sizeof(szSteam2SteamID));

        int i, j;
        for (i = 0; i < g_iPriorityTagsLoaded && bPriorityFound == false; ++i) 
        {
            for (j = 0; j < PRIORITY_TAG_MAX_IDS; ++j) 
            {
                if (g_szSteam2SteamIDs[i][j][0] == '\0') // we reached the end
                    break;
                
                if (StrEqual(g_szSteam2SteamIDs[i][j], szSteam2SteamID)) 
                {
                    char szEngineSteamID[LENGTH_OF_ENGINE_STEAMID + 1];
                    GetClientAuthId(client, AuthId_Engine, szEngineSteamID, sizeof(szEngineSteamID));   
                    FormatEx(szCookieValue, sizeof(szCookieValue), "%s!%s", szCookie2D[GET_RANK], g_szPriorityTags[i]);
                    g_ckTagCookie.SetByAuthId(szEngineSteamID, szCookieValue);
                    bPriorityFound = true;
                    bValidTag = true;
                    break;
                }
            }
        }

        if (!bPriorityFound)
        {
            for (i = 0; i < g_iTagsLoaded; ++i) 
            {
                if (CheckCommandAccess(client, "", ReadFlagString(g_szFlags[i]))) 
                {
                    // We found one, update only the tag
                    char szEngineSteamID[LENGTH_OF_ENGINE_STEAMID + 1];
                    GetClientAuthId(client, AuthId_Engine, szEngineSteamID, sizeof(szEngineSteamID));   

                    FormatEx(szCookieValue, sizeof(szCookieValue), "%s!%s", szCookie2D[GET_RANK], g_szTags[i]);
                    g_ckTagCookie.SetByAuthId(szEngineSteamID, szCookieValue);
                    bValidTag = true;
                    break;
                }
            }
        }
    }
    
    return bValidTag;
}
/**
 * AssignCorrespondingTag
 * 
 * Assigns the first tag that the client has access to and returns true on tag found
 * 
 * @param client            Client index
 * @param args              Number of arguments
 * @param bSetMode          Specifies if we set the cookie or we get it
 */
void CookieActionHandler(int client, int args, bool bSetMode) 
{
    if (args == 0) 
    {
        CReplyToCommand(client, "\x04[Advanced-TagManager]\x01 Usage: sm_advtm_%cetcookie <name/userid> %s", (bSetMode ? 's' : 'g'), (bSetMode ? "<cookie>" : ""));
        return;
    }
    
    char szArg[MAX_NAME_LENGTH + 1];
    char szArg2[COOKIE_MAX_LENGTH];
    if (bSetMode && args == 2) 
        GetCmdArg(2, szArg2, sizeof(szArg2));

    GetCmdArg(1, szArg, sizeof(szArg));

    int target = FindTarget(client, szArg, true, false);
    if (target > 0) 
    {
        if (bSetMode) 
        {
            g_ckTagCookie.Set(target, szArg2);
            CReplyToCommand(client, "\x04[Advanced-TagManager]\x01 Cookie set to: \x04\"%s\"", szArg2);
        } 
        else 
        {
            char szCookieValue[COOKIE_MAX_LENGTH];
            g_ckTagCookie.Get(client, szCookieValue, sizeof(szCookieValue));
            CReplyToCommand(client, "\x04[Advanced-TagManager]\x01 Cookie is: \x04\"%s\"", szCookieValue);
        }
    }
}
